;________________________________________________________________________
;
;                           S A M P L E 1 . A S M  
;________________________________________________________________________
;
; Sample Packet Receiving by Byon Garrabrant  N6BG  byon@mail.com  Apr 99
;________________________________________________________________________
;
;   This software is  available only for non-commercial  amateur radio 
;   or educational  applications.   ALL  other  uses  are  prohibited.  
;   This  software may  be  modified  only  if the  resulting code  be 
;   made available  publicly  and the  original  author given  credit.  
;   Contact the author or TAPR for information on commercial licenses. 
;   (c) Copyright 1999,  Byon Garrabrant,  N6BG.  All rights reserved. 
;________________________________________________________________________

	LIST	P=PIC16F84
	include	"P16F84.INC"
	__CONFIG _HS_OSC & _CP_OFF & _PWRTE_ON & _WDT_OFF
;________________________________________________________________________

	cblock	H'0C'			; variables
		buffer_start:D'30'	; packets up to 11 bytes
		buffer_end
		flags
	endc
	
#define	in_header	flags, 0	
;________________________________________________________________________

	ORG	0			; start code
	goto	main
;________________________________________________________________________

	include	"PACKETI.ASM"		; include packet sending code
;________________________________________________________________________

	cblock				; variables
		serialDelayCount
		serialCount
		serialData
	endc

#define	SERIAL_IN		PORTA, 0
#define	SERIAL_OUT		PORTA, 1

; DELAY_TIME should be (ClockSpeed/4/SerialSpeed-14)/3
; 4MHz 9600 baud would be (4000000/4/9600-14)/3 = 30
; 4MHz 2400=134 4800=65 9600=30 19200=13 38400=4 57600=1
; 10Mhz  4800=169 19200=39

DELAY_TIME	equ	D'39'

serialInit
	bsf	STATUS, RP0		; Select Bank 1
	bsf	SERIAL_IN		; Setup input
	bcf	SERIAL_OUT		; Setup output
	bcf	STATUS, RP0		; Select Bank 0
	return

serialDelay
	movlw	DELAY_TIME		
	movwf	serialDelayCount	; Count down from DELAY_TIME
	nop				; Helps with high speed
serialDelayLoop
	decfsz	serialDelayCount, F	; Countdown
	 goto	serialDelayLoop		; keep counting
	return				; done
	
SerialSend
	movwf	serialData		; save the outgoing byte
	movlw	D'10'
	movwf	serialCount		; set the bit count
	clrc				; prep for start bit
serialTransmitLoop
	bcf	SERIAL_OUT		; assume send low
	skpc				; if sending 0
	 bsf	SERIAL_OUT		; send high
	call	serialDelay		; delay
	setc				; prep for stop bit
	rrf	serialData, F		; roll serialIn bits	
	decfsz	serialCount, F		; if not complete
	 goto	serialTransmitLoop	; loop back for more
	return				; all done
SerialSendCR
	movlw	H'D'
	call	SerialSend
	movlw	H'A'
	goto	SerialSend
	
;________________________________________________________________________

main
	call	serialInit	 	; setup serial
	call	PacketIn_Init		; setup the packet system

	movlw	'-'			; send a simple header
	call	SerialSend
	movlw	'>'
	call	SerialSend
	call	SerialSendCR

	movlw	buffer_start		; reset the buffer
	movwf	FSR

mainLoop
	call	PacketIn_Update		; receive packet
	btfsc	PacketInNewByteFlag	; if received new byte
	 call	packetInByte		;  handle it
	btfsc	PacketInResetFlag	; if packet reset
	 call	packetInReset		;  handle it
	btfsc	PacketInDoneFlag	; if packet done
	 call	packetInDone		;  handle it
	goto	mainLoop		; loop back again
;________________________________________________________________________

packetInByte				; when we receive a byte
	bcf	PacketInNewByteFlag	; clear the flag
	movfw	packetin_byte		; get the byte
	movwf	INDF			; store in buffer
	incf	FSR, F			; inc buffer
	movfw	FSR			
	xorlw	buffer_end
	skpnz				; if end of buffer	
	 decf	FSR, F			;  dec buffer
	return				; done
	
packetInDone				; when packet done
	bcf	PacketInDoneFlag	; clear the flag
	decf	FSR, F			; remove crc
	decf	FSR, F
	movfw	FSR			; save the length
	movwf	buffer_end
	movlw	buffer_start		; reset to start of buffer
	movwf	FSR
	bsf	in_header		; start dumping header
packetInDoneLoop
	clrc
	rrf	INDF, W			; right shift data
	btfss	in_header		; if in header
	 movfw	INDF			;  just load (over)
	call	SerialSend		; send the byte
	btfsc	INDF, 0			; if the data has a low bit
	 bcf	in_header	 	;  we are out of the header
	incf	FSR, F			; bump buffer forward
	movfw	FSR			; check for end of buffer
	xorwf	buffer_end, W
	skpz				; if not at end
	 goto	packetInDoneLoop	;  loop back for more
	call	SerialSendCR		; send CR/LF
	 
	movlw	buffer_start		; reset to the head 
	movwf	FSR			; of the buffer
	return				; done
	
packetInReset
	movlw	'*'
	call	SerialSend
	bcf	PacketInResetFlag	; clear the reset flag
	movlw	buffer_start		; reset to the head 
	movwf	FSR			; of the buffer
	return				; done

	END
;________________________________________________________________________

